/*
 * Created Oct 27, 2007
 * Copyright ThinkTank Mathematics Limited 2007
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER
 *
 * This code is free software; you can redistribute it and/or modify it under the terms of
 * the GNU General Public License version 2 only, as published by the Free Software
 * Foundation. Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE. See the GNU General Public License version 2 for more details (a copy is
 * included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version 2 along with
 * this work; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth
 * Floor, Boston, MA 02110-1301 USA.
 */
package thinktank.httpserver;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLContextSpi;
import javax.net.ssl.SSLEngine;
import javax.net.ssl.SSLParameters;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;

/**
 * This file allows the Java 6 implementation of com.sun.net.httpserver to run under Java
 * 5 by providing convenience methods to use the new {@link SSLParameters} class with the
 * existing {@link SSLEngine} and {@link SSLContext}. This file is GPL as it is
 * distributed with the OpenJDK implementation of com.sun.net.httpserver,
 * {@link javax.net.ssl.SSLParameters}, sun.net.httpserver and the relevant test cases.
 * 
 * @author Samuel Halliday, ThinkTank Mathematics Limited
 */
public class Bridge {

	/**
	 * Replacement for Java 6's <code>SSLContext.getDefaultSSLParameters()</code>. Gets
	 * the parameters from the context. Note that we have to fiddle with the security
	 * model of Java to be able to access a private field and a protected method in order
	 * to do this.
	 * <p>
	 * Returns a copy of the SSLParameters indicating the default settings for this SSL
	 * context.
	 * <p>
	 * The parameters will always have the ciphersuites and protocols arrays set to
	 * non-null values.
	 * 
	 * @param context
	 * @return a copy of the SSLParameters object with the default settings
	 * @throws UnsupportedOperationException
	 *             if the default SSL parameters could not be obtained.
	 */
	public static SSLParameters getDefaultSSLParameters(SSLContext context) {
		try {
			// get the private field "SSLContextSpi contextSpi"
			Field f = SSLContext.class.getDeclaredField("contextSpi");
			// request private access
			f.setAccessible(true);
			// use standard reflection to access the private field
			SSLContextSpi spi = (SSLContextSpi) f.get(context);
			// get the SSLSocket from that with the protected method
			// engineGetSocketFactory
			Method m = SSLContextSpi.class.getDeclaredMethod(
					"engineGetSocketFactory", new Class[0]);
			// request protected access
			m.setAccessible(true);
			SSLSocketFactory factory = (SSLSocketFactory) m.invoke(spi,
					new Object[0]);
			// get an SSLSocket
			SSLSocket socket = (SSLSocket) factory.createSocket();
			SSLParameters params = new SSLParameters();
			// use the SSLSocket to correct the SSLParameters
			params.setCipherSuites(socket.getSupportedCipherSuites());
			params.setProtocols(socket.getSupportedProtocols());
			return params;
		} catch (Exception e) {
			throw new UnsupportedOperationException(
					"Could not obtain parameters", e);
		}
	}

	/**
	 * Replacement for Java 6's <code>SSLEngine.setSSLParameters()</code>. Applies
	 * SSLParameters to this engine.
	 * <p>
	 * This means:
	 * <ul>
	 * <li>if params.getCipherSuites() is non-null, setEnabledCipherSuites() is called
	 * with that value</li>
	 * <li>if params.getProtocols() is non-null, setEnabledProtocols() is called with
	 * that value</li>
	 * <li>if params.getNeedClientAuth() or params.getWantClientAuth() return true,
	 * setNeedClientAuth(true) and setWantClientAuth(true) are called, respectively;
	 * otherwise setWantClientAuth(false) is called.</li>
	 * </ul>
	 * 
	 * @param engine
	 *            the engine
	 * @param params
	 *            the parameters
	 * @throws IllegalArgumentException
	 *             if the setEnabledCipherSuites() or the setEnabledProtocols() call fails
	 */
	public static void setSSLParameters(SSLEngine engine, SSLParameters params) {
		String[] s;
		s = params.getCipherSuites();
		if (s != null) {
			engine.setEnabledCipherSuites(s);
		}
		s = params.getProtocols();
		if (s != null) {
			engine.setEnabledProtocols(s);
		}
		if (params.getNeedClientAuth()) {
			engine.setNeedClientAuth(true);
		} else if (params.getWantClientAuth()) {
			engine.setWantClientAuth(true);
		} else {
			engine.setWantClientAuth(false);
		}
	}

}
